/** 
 *   blob: lectura escritura de los bloques <BLOB>..</BLOB> fuera del
 *   header xml
 *
 *   (c) Copyright 2010 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *   
 **/

#ifndef _BLOB_H
#define _BLOB_H

#include <stdio.h>
#include <unistd.h>

typedef enum {
	BT_UNKNOW=-1,
	BT_RAYINFO,
	BT_RAWDATA,
} BLOB_TYPE;

typedef enum {
	KDP=0,
	dBZ,
	ZDR,
	PhiDP,
	UNKNOW,
	RD_TYPE_SIZE,
} RD_TYPE;


struct blob {
	int id;
	BLOB_TYPE bl_type;
	ssize_t size;
	int compress_flag; /* 0: none, 1: qt(=gzip) */
	unsigned char *data;
	int parse_header_flag;
	int proceced_flag;
	/* rayinfo & rawdata: */
	int rays;
	int depth;
	int slice_refid;
	double posangle;
	/* rawdata only: */
	RD_TYPE type;
	int bins;
	double min;
	double max;
	struct blob *next;
};

#define B16SWAP(v) (((v<<8)&0xff00)|((v>>8)&0x00ff))

#define BLOB_GET_BLOB_TYPE(bl) bl->bl_type
#define BLOB_GET_ID(bl) bl->id
#define BLOB_GET_RAYS(bl) bl->rays
#define BLOB_GET_TYPE(bl) bl->type
#define BLOB_SET_TYPE(bl, new) BLOB_GET_TYPE(bl) = new
#define BLOB_GET_TYPE_STR(bl)				\
	bl->type == KDP?"KDP":bl->type == dBZ?"dBZ":bl->type == ZDR?"ZDR":bl->type == PhiDP?"PhiDP":"unknow"
#define BLOB_GET_BINS(bl) bl->bins
#define BLOB_GET_MIN(bl) bl->min
#define BLOB_SET_MIN(bl, new) BLOB_GET_MIN(bl) = new
#define BLOB_GET_MAX(bl) bl->max
#define BLOB_SET_MAX(bl, new) BLOB_GET_MAX(bl) = new
#define BLOB_GET_DEPTH(bl) bl->depth
#define BLOB_SET_DEPTH(bl, new) BLOB_GET_DEPTH(bl) = new
#define BLOB_GET_DATA(bl) bl->data
#define BLOB_SET_DATA(bl, newdata) BLOB_GET_DATA(bl) = (unsigned char *) newdata
#define BLOB_GET_DATA_UINT16(bl, i) (*(((uint16_t *) BLOB_GET_DATA(bl))+i))
#define BLOB_GET_DATA_UINT8(bl, i) (*(((uint8_t *) BLOB_GET_DATA(bl))+i))
#define BLOB_GET_DATA_SIZE(bl) bl->size
#define BLOB_SET_DATA_SIZE(bl, new) BLOB_GET_DATA_SIZE(bl) = new
#define BLOB_GET_POSANGLE(bl) bl->posangle
#define BLOB_GET_SLICE_REFID(bl) bl->slice_refid

#define BLOB_DEPTH_FACTOR(bl)\
	(BLOB_GET_DEPTH(bl) == 8?254.0:BLOB_GET_DEPTH(bl) == 16?65534.0:0.0)

#define BLOB_GET_DATA_UINT(bl, i)				\
	(BLOB_GET_DEPTH(bl) == 16?B16SWAP(BLOB_GET_DATA_UINT16(bl, i)):	\
	 BLOB_GET_DEPTH(bl) == 8?BLOB_GET_DATA_UINT8(bl, i):-1)

#define BLOB_GET_VALUE_SCALE(bl, i)					\
	(BLOB_GET_DATA_UINT(bl, i) == 0?0:		\
	(double) (BLOB_GET_DATA_UINT(bl, i) - 1) *			\
	 (BLOB_GET_MAX(bl) - BLOB_GET_MIN(bl)) / BLOB_DEPTH_FACTOR(bl) + \
	 BLOB_GET_MIN(bl))

#define BLOB_GET_VALUE_SCALE_XY(bl, x, y)		\
	BLOB_GET_VALUE_SCALE(bl, (y * BLOB_GET_BINS(bl) + x))
// No!	BLOB_GET_VALUE_SCALE(bl, (y * BLOB_GET_RAYS(bl) + x))

#define BLOB_IS_NO_DATA_XY(bl, x, y)		\
	(BLOB_GET_DATA_UINT(bl, (y * BLOB_GET_BINS(bl) + x)) == 0?1:0)

#define BLOB_RAYINFO(bl, i)				  \
	((double) B16SWAP(BLOB_GET_DATA_UINT16(bl, i)) * 360.0 / 65536.0 )

void free_blob_all(struct blob *bl);
int blob_uncompress(struct blob *bl);
int blob_compress(struct blob *bl);
 /* blob_get: cal previus to <BLOB blobid="0" size="737"
  * compression="qt"> */
int blob_get_file(const char *fname, struct blob **abl);
int blob_print(FILE *fp, struct blob *bl, int compress_flag);
struct blob *blob_get_blob_id(struct blob *bl, int id);
/*
void blob_set_rays(struct blob *bl, int rays);
void blob_set_depth(struct blob *bl, int depth);
void blob_set_rdtype(struct blob *bl, RD_TYPE type);
void blob_set_bins(struct blob *bl, int bins);
void blob_set_min(struct blob *bl, int min);
void blob_set_max(struct blob *bl, int max);
void blob_header_completed(struct blob *bl);
*/
struct blob *blob_set_rawdata(struct blob *bl, const char **val);
struct blob *blob_set_rayinfo(struct blob *bl, const char **val);


#endif /* ifndef _BLOB_H */
