/** 
 *   blob2txt: Transformar los bloques binarios BLOB a texto.
 *
 *   (c) Copyright 2009 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <getopt.h>
#include <fcntl.h>
#include <my_stdio.h>
#include "version.h"
#include "my_xcomp.h"

#define RAW_8  0xFF08
#define TXT_4  0x0004
#define TXT_8  0x0008
#define TXT_16 0x0010


/*
rawdata blobid="1" rays="361" type="KDP" bins="480" min="-18.8" max="23.7" depth="16" 
 (65535 -1)*(23.7+18.8)/65534 - 18.8
 printf ("%f ",((double) *i - 1.0) *(23.7+18.8)/ ((double) 0xfffe) + 18.8 );
*/
/*
cat 2009111321400200KDP.vol |awk 'BEGIN{dump=0} /.\/BLOB./{dump=0} {if(dump) print $0;} /.BLOB blobid=\"1\".*./{dump = 1}'
 */


const char *name_definition = {"Binary BLOB Tag to text converter"};
static const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - verbose mode (incremental).",
#       endif
	"  -d bits\t - Number of bits used to describe one data point (4, 8 or 16).",
	"  --kdp\t\t - KDP scale conversion.",
	"  --dbz\t\t - dbZ scale conversion.",
	"  --rayinfo\t - Ray Info scale conversion.",
	"  --emin\t - Set minimum for scaling range.",
	"  --emax\t - Set maximum for scaling range.",
	"  --binary-out\t - Binary output (de/compress).",
	"  --compress\t - Compress imply --binary-out.",
        "  -V\t\t - Show program version.",
//        "  --link\t - Make symbolic links for each command.",
//        "  --unlink\t - Remove symbolic links for commands.",
        "  -h [--help]\t - this help !!!.",
        ""
};

void version(char *program_name)
{
        fprintf(stderr,"%s: ", name_definition);
        fprintf(stderr,"%s version %s\n", program_name, VERSION);
}


void usage(char *program_name)
{
        version(program_name);
        fprintf(stderr,
#               ifdef VERBOSE_MODE
 "Usage: \t%s [-vhV][-d bits][--emin val --emax val][--kdp|--dbz|--raiyangle|[--binary-out][--compress]][--help]\n",
#                else
 "Usage: \t%s [-hV][-d bits][--emin val --emax val][--kdp|--dbz|--raiyangle|[--binary-out][--compress]][--help]\n",
#               endif
                program_name);
}


void help()
{
        int i=0;

        while((char) *help_text[i] != 0)
                fprintf(stderr,"%s\n",help_text[i++]);
        fprintf(stderr,"\n");
}


int print_kdp(unsigned int value, double min, double max)
{
	if ( value )
		printf("%10.6f\n", 
		       (double) (value - 1) * (max - min) / 65534.0 + min);
	else
		printf("-\n");

	return 0;
}


int print_rayinfo(unsigned int value, double min, double max)
{
	printf("%f\n", (double) value * (max - min) / 65536.0 + min);

	return 0;
}


int print_dbz(unsigned int value, double min, double max)
{
	if ( value )
		printf("%5.1f\n", 
		       (double) (value - 1) * (max - min) / 254.0 + min);
	else
		printf("-\n");

	return 0;
}


int get_data_from_fd(int fd, unsigned char **data)
{
	int l = -1;
        int n = 0, size = 480 * 2, offset = 0;
        char *buff = NULL;

        if ((buff = malloc (size)) == NULL) return l;
        while (1) {
                /* Try to read in the allocated space. */
		do {
			n = read(fd, buff+offset, size-offset);
			offset += n;
		} while(n !=0 && offset < size);

		if (offset == size) {
			size *= 2;  /* twice the old size */
                        if ((buff = realloc (buff, size)) == NULL)
				return -1;
		} else {
                        break;
		}
	}
	if ( buff !=  NULL)
		*data = (unsigned char *) buff;	
	
	return offset;
}


int main(int argc, char **argv)
{
	size_t size_out, size_in;
	size_t j;
	unsigned char *data_out;
	int bits = 16;
        int ret = 0;
        int op;
	char *file_in;
	int fd_in = -1;
	int binary_out_flag = 0;
	int uncompress_flag = 1;
	int kdp_flag = 0;
	int dbz_flag = 0;
	int rayinfo_flag = 0;
	int emin_flag = 0;
	int emax_flag = 0;
	double emin = 0.0;
	double emax = 0.0;
	unsigned int value;
        char *program_name;
        char *cp;
	unsigned char *data_in;
        struct option const longopts[] =
        {
                {"help", 0, 0, 'h'},
                {"verbose", 0, 0, 'v'},
                {"link", 0, 0, 201},
                {"unlink", 0, 0, 202},
		{"kdp", 0, 0, 203},
		{"dbz", 0, 0, 204},
		{"rayinfo", 0, 0, 205},
		{"emin", 1, 0, 206},
		{"emax", 1, 0, 207},
		{"binary-out", 0, 0, 208},
		{"compress", 0, 0, 209},
                {0,  0,  0,  0}
        };

        /* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];

        verbose_flag = PERR;
        while ((op = getopt_long(argc, argv, "vhVd:",longopts, 0)) != EOF)
                switch( op ){
                case 203:
			kdp_flag = 1;
			bits = 16;
                        break;
                case 204:
			dbz_flag = 1;
			bits = 8;
                        break;
                case 205:
			rayinfo_flag = 1;
			bits = 16;
                        break;
		case 206:
			emin = atof(optarg);
			emin_flag = 1;
			break;
		case 207:
			emax = atof(optarg);
			emax_flag = 1;
			break;
		case 208:
			binary_out_flag = 1;
			break;
		case 209:
			uncompress_flag = 0;
			binary_out_flag = 1;
			break;
                case 'd':
			bits = atoi(optarg);
                        break;
                case 'v':
                        verbose_flag = (verbose_flag << 1) | 1;
                        break;
                case 'V':
                        version(program_name);
                        exit(1);
                        break;
                case 201:
                        //link_commands_to(argv[0] /* program_name */ );
                        exit(0);
                case 202:
                        //un_link_commands();
                        exit(0);
                case 'h':
			usage(program_name);
                        help();
                        exit(1);
                default:
                        usage(program_name);
                        exit(1);
                }
	if ((kdp_flag+dbz_flag+rayinfo_flag) > 1){
		PVERB(PERR,"Only one conversion flag epesify.");
		exit(3); /* bad parameter */
	}
	if(kdp_flag) {
		if ( !emax_flag) {
			PVERB(PWARN,
			      "--emax value: not espesify, asuming default");
			emax = 20.0;
		}
		if ( !emin_flag) {
			PVERB(PWARN,
			      "--emin value: not espesify, asuming default");
			emin = -20.0;
		}
	} else if(dbz_flag) {
		if ( !emax_flag) {
			PVERB(PWARN,
			      "--emax value: not espesify, asuming default");
			emax = 95.5;
		}
		if ( !emin_flag) {
			PVERB(PWARN,
			      "--emin value: not espesify, asuming default");
			emin = -31.5;
		}
	} else if(rayinfo_flag) {
		if ( !emax_flag) {
			PVERB(PWARN,
			      "--emax value: not espesify, asuming default");
			emax = 360.0;
		}
		if ( !emin_flag) {
			PVERB(PWARN,
			      "--emin value: not espesify, asuming default");
			emin = 0.0;
		}
	}
	file_in = argv[optind];
	if (binary_out_flag) {
		bits = RAW_8;
	}

	if (file_in == NULL || *(file_in) == '-') {
		fd_in = 0;
	} else {
		fd_in = open(file_in, O_RDONLY);
	}
	if (fd_in < 0) {
		PVERB(PERR,"Bad file name: %s", file_in);
		exit(2);
	}
	
	size_in = get_data_from_fd(fd_in, &data_in);
	if ( *(data_in+(size_in-1)) == 0x0a ) {
		PVERB(PINFO,"data_in end with 0x0a.");
		size_in--;
	}

	if (uncompress_flag) {
		data_out = (unsigned char *) 
			my_uncomp((void *) data_in, size_in, &size_out);
	} else {
		data_out = (unsigned char *)
			my_comp((void *) data_in, size_in, &size_out);
	}
	free(data_in);
	if (data_out != NULL) {
		switch(bits) {
		case TXT_16:
			for (j=0 ;j < size_out;j+=2){
				value = data_out[j] <<8 | data_out[j+1];
				if (kdp_flag)
					print_kdp(value, emin, emax);
				else if(rayinfo_flag)
					print_rayinfo(value, emin, emax);
				else 
					printf("%d\n", value );
			}
			break;
		case TXT_8:
			for (j=0 ;j < size_out;j++){
				if (dbz_flag)
					print_dbz(*(data_out+j), emin, emax);
				else 
					printf("%d\n", *(data_out+j));
			}
			break;
		case TXT_4: /* Creo que  no lo usan... */
			PVERB(PWARN,"Bits size %d not tested.", bits);
			for (j=0 ;j < size_out;j++){
				/* FIXME: el orden... */
				printf("%d\n", *(data_out+j) >> 4 );
				printf("%d\n", *(data_out+j) & 0x0f);
			}
			break;

		case RAW_8:
			for (j=0 ;j < size_out;j++){
				printf("%c", *(data_out+j));
			}
			break;
		default:
			PVERB(PERR,"Invalid bits size: -d %d", bits);
		}
					
		free(data_out);
	}

    return ret;
}

