/** 
 *   get_point: Extraer los puntos de las coordenadas lat, lon
 *
 *   (c) Copyright 2010 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *   
 **/

#define _GNU_SOURCE

#include <my_stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>

#include "blob.h"
#include "my_xcomp.h"
#include "get_point.h"
#include "ang_tools.h"
#include "geo.h"



static int mod(int module, int x)
{
	while (x < 0) x += module;
	while (x > module) x -= module;
			 
	return x;
}
 

#if 0
static void print_data_blob_9(FILE *fp, struct blob *bl, int d, int a)
{
	int i, j;
	int rd, ra;

	// 1, 2
	for (i = a - 1; i < a + 2; i++) {
		ra = mod(BLOB_GET_RAYS(bl), i);
		for(j = d - 1; j < d + 2; j++) {
			rd = mod(BLOB_GET_BINS(bl), j);
			if (BLOB_GET_DATA_UINT(bl, (ra * BLOB_GET_BINS(bl) + rd)) == 0 )
				fprintf(fp, "%6s ", "-"); 
			else 
				fprintf(fp, "%6.3f ", 
					BLOB_GET_VALUE_SCALE_XY(bl, rd, ra));
		}
	}
	fprintf(fp, "\n");
	
	return;
}
#endif


static void print_data_blob(FILE *fp, struct blob *bl, struct gp_param *p,
			    int d, int a)
{
	int i, j;
	int ra;
	int f = (p->pixel_grid - 1) >> 1;
	int t = (p->pixel_grid + 1) >> 1;
	int pix = 1;
	int is_no_data;

	for (i = a - f; i < a + t; i++) {
		ra = mod(BLOB_GET_RAYS(bl), i);
		for(j = d - f; j < d + t; j++) {
			is_no_data = BLOB_IS_NO_DATA_XY(bl, j, ra);
			if ( !(p->no_data_out_flag == 0 && is_no_data == 1) ) {
				if (p->time_flag)
					fprintf(fp,"%04d%02d%02d%02d%02d ",
						p->tm->tm_year + 1900, 
						p->tm->tm_mon + 1, 
						p->tm->tm_mday,
						p->tm->tm_hour, p->tm->tm_min);
				if (p->posangle_flag)
					fprintf(fp, "%4.1f ", 
						BLOB_GET_POSANGLE(bl));
				if (p->pix_num_flag) {
					fprintf(fp, "%2d ", pix++);
					if ( j<0 || j >= BLOB_GET_BINS(bl) || 
					     is_no_data ) {
						fprintf(fp, "%6s ", p->no_data); 
					} else {
						fprintf(fp, "%6.3f ", 
							BLOB_GET_VALUE_SCALE_XY(
								bl, j, ra));
					}
					fprintf(fp, "\n");
				}
			}
		}
	}

	return;
}


static int rayinfo_find_angle(struct blob *bl, double angle)
{
	int index = -1;
	double start_angle;

	if ( BLOB_GET_BLOB_TYPE(bl) != BT_RAYINFO ) {
		PVERB(PWARN,"Caling with invalid struct blob (type: %d)", 
		      BLOB_GET_BLOB_TYPE(bl));
		return index;
	}
	start_angle = BLOB_RAYINFO(bl, 0);
	if ( (start_angle - angle) < 0.0) {
		index = (int) ceil(angle - start_angle) ;
	}else if ( (start_angle - angle) > 0.0) {
		index = (BLOB_GET_RAYS(bl)-1) + (int) ceil(angle - start_angle) ;
	}
	//PDBG("Angles: %f (at %d) %f", BLOB_RAYINFO(bl, index), index, angle - BLOB_RAYINFO(bl, index));

	return index;
}


int blob_find_pos(FILE *fp, struct blob *bl, struct gp_param *p,
		  double distance, double bearing)
{
	int count = 0;
	struct blob *blp;
	int slice_refid = -1;
	int aindex = -1;
	int dindex = -1;

	for(blp = bl; blp != NULL; blp = blp->next) {
		if( blp->compress_flag == 1) {
			blob_uncompress(blp);
		}
		if (BLOB_GET_BLOB_TYPE(blp) == BT_RAYINFO) {
			aindex = rayinfo_find_angle(blp, bearing);
			if (aindex >= 0) {
				slice_refid = BLOB_GET_SLICE_REFID(blp);
			} else {
				slice_refid = -1;
			}
		}else if (BLOB_GET_BLOB_TYPE(blp) == BT_RAWDATA &&
			  BLOB_GET_SLICE_REFID(blp) == slice_refid) {
			dindex = (int) ceil(
				distance / cos(BLOB_GET_POSANGLE(blp)*_DEG2RAD)*
				(double) BLOB_GET_BINS(blp) /
				(double) p->stop_range);
			print_data_blob(fp, blp, p, dindex, aindex);
		} else {
			PVERB(PWARN,"Unknow blob type: %d", 
			      BLOB_GET_BLOB_TYPE(blp));
		}
	}
	
	return count;
}


int blob_dump_pos(FILE *fp, struct blob *bl, struct gp_param *p,
		  double center_lat, double center_log)
{
	int count = 0;
	struct blob *blp;
	struct blob *bl_ri = NULL;
	double ray_info;
	double cos_f;
	int slice_refid = -1;
	int i, j;
	double tlat = 0.0, tlon=0.0;
	int is_no_data;

	for(blp = bl; blp != NULL; blp = blp->next) {
		if( blp->compress_flag == 1) {
			blob_uncompress(blp);
		}
		if (BLOB_GET_BLOB_TYPE(blp) == BT_RAYINFO) {
			slice_refid = BLOB_GET_SLICE_REFID(blp);
			bl_ri = blp;
		}else if (BLOB_GET_BLOB_TYPE(blp) == BT_RAWDATA &&
			  BLOB_GET_SLICE_REFID(blp) == slice_refid) {
			cos_f = cos(BLOB_GET_POSANGLE(blp)* _DEG2RAD);
			for(i = 0;i<BLOB_GET_RAYS(blp);i++){
				ray_info = BLOB_RAYINFO(bl_ri, i);
				for(j=0;j<BLOB_GET_BINS(blp);j++) {
					is_no_data = BLOB_IS_NO_DATA_XY(blp,j,i);
					if ( !(p->no_data_out_flag == 0 && 
					       is_no_data == 1) ) {
#if 0	
						fprintf(fp,"%f %f %f ",
							ray_info, 
							(double)j * cos_f *
							(double) p->stop_range /
							(double) BLOB_GET_BINS(
								blp),
							BLOB_GET_POSANGLE(blp));
#else	
						geo_dist_ber2dest_point
							(p->r_lat, p->r_lon,
							 ray_info * _DEG2RAD ,
							 (double)j * cos_f * 
							 (double) p->stop_range /
							 (double) BLOB_GET_BINS(
								 blp),
							 &tlat, &tlon);
						fprintf(fp,"%8.4f %8.4f %4.1f ",
							tlat/ _DEG2RAD, tlon / 
							_DEG2RAD,
							BLOB_GET_POSANGLE(blp));
#endif
						if ( is_no_data )
							fprintf(fp, "%6s\n", 
								p->no_data); 
						else 
							fprintf(fp, "%6.3f\n", 
								BLOB_GET_VALUE_SCALE_XY(blp, j, i));
					}
				}
			}
		} else {
			PVERB(PWARN,"Unknow blob type: %d", 
			      BLOB_GET_BLOB_TYPE(blp));
		}
	}
	
	return count;
}


