/** 
 *   xml_proc: read, proces and write xml data file. 
 *
 *   (c) Copyright 2010 Federico Bareilles <bareilles@gmail.com>,
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *   
 **/

#ifndef _XML_PROC_H
#define _XML_PROC_H

#include <expat.h>
#include "blob.h"
#include "radar.h"

typedef enum {
	STORE_UNKNOW = 0,
	STORE_INT,
	STORE_DOUBLE,
	STORE_CHAR,
} STORE_TYPE;

/* xml proc user data: */
typedef struct {
	FILE *fpo;
	int add_to_out_flag;
	struct blob *bl;
	int inivit_out_flag; /* 0: never out, 1: use out_flag */ 
	int out_flag; /* 1: out, 0: not out */ 
	void *parameters;
	int (*proc_rawdata) (struct blob *bl, void *parameters);
	radar_info *ri;
	double current_posangle;
	int stoprange;
	int current_refid;
	void *store_value;
	STORE_TYPE store_type;
} xmlp_udata; 


#define XMLP_GET_FPO(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->fpo
#define XMLP_SET_FPO(parser, fp)				\
	XMLP_GET_FPO(parser) = fp
#define XMLP_GET_OUT_FLAG(parser)					\
	((xmlp_udata *) XML_GetUserData(parser))->out_flag
#define XMLP_SET_OUT_FLAG(parser, flag)					\
	XMLP_GET_OUT_FLAG(parser) = flag
#define XMLP_GET_INIVIT_OUT_FLAG(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->inivit_out_flag
#define XMLP_IF_OUT_FLAG(parser)					\
	XMLP_GET_INIVIT_OUT_FLAG(parser) == 1?0:XMLP_GET_OUT_FLAG(parser)
#define XMLP_SET_INIVIT_OUT_FLAG(parser, flag)	\
	XMLP_GET_INIVIT_OUT_FLAG(parser) = flag
#define XMLP_GET_BLOB(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->bl
#define XMLP_SET_BLOB(parser, blob)				\
	XMLP_GET_BLOB(parser) = blob
#define XMLP_GET_PROC_PARAMETERS(parser)			\
	((xmlp_udata *) XML_GetUserData(parser))->parameters
#define XMLP_SET_PROC_PARAMETERS(parser, param)				\
	XMLP_GET_PROC_PARAMETERS(parser) = param
#define XMLP_GET_PROC_RAWDATA(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->proc_rawdata
#define XMLP_SET_PROC_RAWDATA(parser, func)				\
	XMLP_GET_PROC_RAWDATA(parser) = func
#define XMLP_GET_CURRENT_POSANGLE(parser)	\
	((xmlp_udata *) XML_GetUserData(parser))->current_posangle
#define XMLP_GET_CURRENT_REFID(parser) \
	((xmlp_udata *) XML_GetUserData(parser))->current_refid
#define XMLP_GET_STORE_TYPE(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->store_type
#define XMLP_GET_STORE_VALUE(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->store_value
#define XMLP_GET_STOPRANGE(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->stoprange
#define XMLP_GET_ADD_OUT_FILE(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->add_to_out_flag


/* For radar_info structure: */
#define XMLP_GET_RADAR_INFO(parser)				\
	((xmlp_udata *) XML_GetUserData(parser))->ri
#define XMLP_SET_RADAR_INFO(parser, rad)	\
	XMLP_GET_RADAR_INFO(parser) = rad
#define XMLP_GET_LAT(parser)			\
	RI_GET_LAT(XMLP_GET_RADAR_INFO(parser))
#define XMLP_GET_LON(parser)			\
	RI_GET_LON(XMLP_GET_RADAR_INFO(parser))
#define XMLP_GET_ALT(parser)			\
	RI_GET_ALT(XMLP_GET_RADAR_INFO(parser))
#define XMLP_GET_RIID(parser)			\
	RI_GET_ID(XMLP_GET_RADAR_INFO(parser))
#define XMLP_GET_TIME(parser)			\
	RI_GET_TIME(XMLP_GET_RADAR_INFO(parser))
#define XMLP_GET_TM(parser)			\
	RI_GET_TM(XMLP_GET_RADAR_INFO(parser))
#define XMLP_GET_BEAMWIDTH(parser)			\
	RI_GET_BEAMWIDTH(XMLP_GET_RADAR_INFO(parser))

XML_Parser xmlp_init(void); 
int xmlp_set_FILE_out(XML_Parser parser, FILE *fp);
int xmlp_open_file_out(XML_Parser parser, char *fname);
FILE *xmlp_get_FILE_out(XML_Parser parser);
int xmlp_close_file_out(XML_Parser parser);
void xmlp_free(XML_Parser parser);

int xmlp_run_on(XML_Parser parser, char *fname);

#endif /* _XML_PROC_H */
